function TagFactory () {
};


TagFactory.prototype.createTag = function(tagValue) {
    var tag = document.createElement('DIV');

    tag.style.display = "none";
    tag.innerHTML = tagValue;
    return tag;
};



/* this is the way we create tags for ipodradio */
function TagFactoryIPodRadio(url) {
    this.url = url;
};

TagFactoryIPodRadio.prototype = new TagFactory;

TagFactoryIPodRadio.prototype.createTag = function (tagValue) {
    var tag = document.createElement("span");
    var link= document.createElement("A");
    
    link.innerHTML = tagValue;
    link.href = this.url + tagValue;
    link.className = "link_header_01";
    tag.appendChild(link);
    return tag;    
};



/* this is the way we create tags for lvision */
function TagFactoryLVision(url) {
    this.url = url;
    this.useDeleteIcon = false;
};

TagFactoryLVision.prototype = new TagFactory;


TagFactoryLVision.prototype.createTag = function (tagValue) {
    var tag = document.createElement("span");
    var link= document.createElement("A");
    var del;
    
    link.innerHTML = tagValue;
    link.href = this.url + tagValue;
    link.className = "link_tg";

    tag.appendChild(link);
    return tag;    
};



/* 
 * Class TagsArea
 */

/* Tasg area: an object binded to some container where tags are displayed */
/* iod - some object id passed to server to bind a the tag */
function TagsArea (area,oid,delTags) {
    this.canvas = $(area);
    this.tagFactory = null;
    this.id = area;
    this.tags = new Array();
    this.delim = ',  ';
    this.oid = oid;
    this.delTags = delTags;
    this.tagEditor = null;
};



/* */
TagsArea.prototype.setTagEditor = function(editor) {
    this.tagEditor = editor;
};



/* */
TagsArea.prototype.setTagFactory = function (factory) {
    this.tagFactory = factory;
};



/* */
TagsArea.prototype.addTag = function(tagValue) {

    if (this.tagFactory == null) {
        this.tagFactory = new TagFactory();
    }
        
    var tag = this.tagFactory.createTag(tagValue);

    if (this.delTags) {
	this._createDeleteTagLink(tag,tagValue);
    }
    
    if (this.tags.length != 0) {
        this.canvas.appendChild(document.createTextNode(this.delim));
    }    
    
    this.canvas.appendChild(tag);
/*    new Effect.Pulsate(tag);  */
    this.tags.push(tagValue);
    
};



/* */
TagsArea.prototype._createDeleteTagLink = function(tag,tagVal) {
    var self = this;

    if (this.tagEditor === null) {
	return;
    }
    
    var link = document.createElement('A');
    var img  = document.createElement('IMG');
    
  img.src = '/img/deltag.gif';
	img.border = 0;
	img.style.verticalAlign = 'middle';
	img.style.marginLeft = '2';
	img.style.marginRight = '4';
	
	link.href = 'javascript:void(0)';
	link.title = link.alt = img.alt = ' ';
	link.appendChild(img); 

	link.onclick = function() {
	    if (confirm('  ' + tagVal + '?')) {
		self.tagToDelete = tag;
		self.tagValToDelete = tagVal;
		self.tagEditor.deleteTag(self,tagVal);
	    }
	};
	
	tag.appendChild(link);   
};



/* */
TagsArea.prototype.deleteTag = function() {
    var tag = this.tagToDelete;
/*    new Effect.Fade(tag); */
    tag.style.display = 'none';
};



/* create all tags and draw them */
TagsArea.prototype.redraw = function() {
    this.canvas.innerHTML = '';
    
    if (this.tagFactory == null) {
        this.tagFactory = new TagFactory();
    }

    for (var i = 0; i < this.tags.length; i++) {
        var tag = this.tagFactory.createTag(this.tags[i]);
    
        if (this.delTags) {
	    this._createDeleteTagLink(tag,this.tags[i]);
        }

        this.canvas.appendChild(tag);

        if (i != this.tags.length - 1) {
            this.canvas.appendChild(document.createTextNode(this.delim));
        }
    }
};


/* 
 * Class TagEditor
 */

/* tag editor constutor */
function TagEditor(editorDiv, input, addBtn, closeBtn, surl, addurl,delurl) {
    this.canvas = $(editorDiv);
    this.parent = this.canvas.parentNode;
    this.visible= false;
    this.canvas.style.display = "none";

    /* some controls */
    this.input          = $(input);
    this.closeButton    = $(closeBtn);
    this.addButton      = $(addBtn);   
    this.suggestURL     = surl;
    this.addURL         = addurl;
    this.deleteURL 	= delurl;

    this._init();    
};




/* private method. init some vars */
TagEditor.prototype._init = function () {
    var self = this;
    
    this.closeButton.onclick = function () {
        /* new Effect.Fade(self.canvas); */
	self.canvas.style.display = 'none';
        self.visible = false;
    };
    
    this.addButton.onclick = function () {
        self.addTag();
    };


    this.tagsArea = null;
    this.sProvider = new SuggestionProvider(this.suggestURL);
    this.sControl  = new AutoSuggestControl(this.input, this.sProvider);

    this.sControl.handleEnterPressed = function () {
        self.addTag();
    };

    this.xmlHttp = Ajax.getTransport();
};

/* */



/* show tag editor inside some container (div etc.) */
TagEditor.prototype.showIn = function(target, oTagsArea) {

    if (this.visible) {
        this.canvas.style.display = "none";
    }
            
    $(target).appendChild(this.canvas);
    
    this.input.value = '';
    
/*    new Effect.Appear(this.canvas); */
    this.canvas.style.display = 'block';
    this.visible = true;
    this.tagsArea = oTagsArea;
};




/* user input a tag. add it */
TagEditor.prototype.addTag = function() {
    var tagValue = this.input.value;
    
    if (tagValue.length == 0) {
        return;
    }
    
    var http = this.xmlHttp;
    var self = this;
    
    if (http.readyState != 0) {
        http.abort();
    }
    
    var url = this.addURL;
    var r1  = url.replace('%%TAGVAL%%', escape(tagValue));
    var r2  = r1.replace('%%OBJECTID%%', this.tagsArea.oid);
    
    http.open("GET", r2, true);
    http.onreadystatechange = function () {
    
            if (http.readyState != 4)
                return;

            self.input.value = '';        
            
            if (http.status != '200') 
                return;
            
            if (http.responseText.length == 0)
                return;
            
            self.tagsArea.addTag(http.responseText);
            self.input.focus();
    };

    http.send(null);
    return;
};



TagEditor.prototype.deleteTag = function(area,tagVal) {
    var http = this.xmlHttp;
    var self = this;
    
    if (http.readyState != 0) {
        http.abort();
    }
    
    var url = this.deleteURL;
    var r1  = url.replace('%%TAGVAL%%', escape(tagVal));
    var r2  = r1.replace('%%OBJECTID%%', area.oid);
    
    http.open("GET", r2, true);
    http.onreadystatechange = function () {
    
            if (http.readyState != 4)
                return;

            if (http.status != '200') 
                return;
            
            if (http.responseText.length == 0)
                return;
            
	    if (http.responseText == 'OK')
    	        area.deleteTag();
    };

    http.send(null);
    return;
    
};




